#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .models import *
import datetime
import sense_core as sd
from . import email_base, utils
from .share_interface import Share
from .check_db_exist import check_db_exist

sd.log_init_config(root_path=sd.config('log_path'))


class MonitEmialSender(object):
	
	def _send_err_email(self, module, title, subject = "网站日志监控", receiver_email=None):
		db = check_db_exist()
		module_config = {}
		email_list = []
		if receiver_email:
			if type(receiver_email) == list:
				email_list = receiver_email
			elif type(receiver_email) == str:
				email_list = [receiver_email]
		else:
			if not db:
				return None
			else:
				module_config = Share.get_module_config(module)
				email_list = Share.get_contact_list(module_config['user'], 'email')
		send_res = self._send_email(module, title, subject, email_list)
		suc_email = []
		fail_email = []
		if not send_res or send_res == '':
			sd.log_info('[*] Module ' + module + ' errors email has send failed.')
		else:
			suc_email, fail_email = self._check_send_result(send_res)
			if fail_email and len(fail_email) != 0:
				suc_list, fail_list = self._resend_email(module, fail_email, subject, title)
				suc_email.extend(suc_list)
				fail_email = fail_list
		if not db:
			sd.log_info('The success send email is:{}, the failed send email is:{}'.format(suc_email, suc_email))
		else:
			try:
				self._update_email_db(module, title, str(module_config['user']), suc_email, fail_email)
			except Exception as e:
				self._update_email_db(module, title, str([]), suc_email, fail_email)

	
	def _send_email(self, module, title, subject, email_list=None):
		_now = datetime.datetime.now()
		msg_res = self._send_email_detail(module, email_list, subject, title)
		return msg_res
	
	def _get_last_send_time(self, module_name):
		email_send_log = EmailSendLog()
		session = email_send_log.get_session()
		email_send = session.query(EmailSendLog).filter(EmailSendLog.module == module_name).order_by("time").all()
		if email_send:
			last_time = email_send[-1].time
		else:
			last_time = datetime.datetime.strptime("2019-01-01 00:00:00", "%Y-%m-%d %H:%M:%S")
		email_send_log.close_session(session)
		return last_time
	
	def _check_send_result(self, send_res):
		suc_email, fail_email = [], []
		if send_res['code'] == 'OK':
			suc_email = send_res['addr']
		elif send_res['code'] == 'Failed':
			suc_email = send_res['suc_list']
			fail_email = send_res['failed_list']
		return suc_email, fail_email
	
	def _update_email_db(self, module_name, content, uid, success_email, fail_email):
		email_send_log = EmailSendLog()
		session = email_send_log.get_session()
		_now = datetime.datetime.now()
		if success_email and len(success_email) != 0:
			for item in success_email:
				bulk_object = EmailSendLog(module=module_name, content=content, uids=uid, email=item,
										   status='success', time=_now)
				session.add(bulk_object)
		if fail_email and len(fail_email) != 0:
			for item in fail_email:
				bulk_object = EmailSendLog(module=module_name, content=content, uids=uid, email=item,
										   status='failed', time=_now)
				session.add(bulk_object)
		try:
			session.commit()
		except Exception as e:
			sd.log_warn('update err email db occur error:{0}'.format(e))
		finally:
			email_send_log.close_session(session)
	
	def _send_email_detail(self, module, email_list, subject, title):
		if not email_list or len(email_list) == 0:
			sd.log_info('The receiver email is empty')
			return {}
		_res = email_base.send_email(module, email_list, subject, title)
		res = self._is_mail_failed(_res)
		if res:
			_res = utils.build_dic(code='Failed', flag='email', failed_list=res)
		else:
			_res = utils.build_dic(code='OK', flag='email', addr=email_list)
			sd.log_info('email send successful!!!')
		return _res
	
	def _resend_email(self, module, fail_email, subject, title):
		success_list = []
		fail_list = fail_email
		if fail_email and len(fail_email) != 0:
			i = 1
			while i < 3:
				sd.log_info('email send fail, the failed email address is {}, now try {} resend'.format(fail_list, i))
				email_res = self._send_email_detail(module, fail_email, subject, title)
				if email_res['code'] == 'OK':
					success_list.extend(fail_email)
					fail_list = []
					break
				elif email_res['code'] == 'Failed':
					fail_list = email_res['failed_list']
					success_list.extend(email_res['suc_list'])
				i += 1
		return success_list, fail_list
	
	def _is_mail_failed(self, res):
		if not isinstance(res, dict):
			return []
		if len(res) == 0:
			return False
		return list(res.keys())



#!/usr/bin/python
# -*- coding:utf-8 -*-

import uuid
from aliyunsdkcore.client import AcsClient
from aliyunsdkcore.profile import region_provider
from aliyunsdkcore.request import RpcRequest
import json
import sense_core as sd

class SendSmsRequest(RpcRequest):
    def __init__(self):
        RpcRequest.__init__(self, 'Dysmsapi', '2017-05-25', 'SendSms')

    def get_TemplateCode(self):
        return self.get_query_params().get('TemplateCode')

    def set_TemplateCode(self,TemplateCode):
        self.add_query_param('TemplateCode',TemplateCode)

    def get_PhoneNumbers(self):
        return self.get_query_params().get('PhoneNumbers')

    def set_PhoneNumbers(self,PhoneNumbers):
        self.add_query_param('PhoneNumbers',PhoneNumbers)

    def get_SignName(self):
        return self.get_query_params().get('SignName')

    def set_SignName(self,SignName):
        self.add_query_param('SignName',SignName)

    def get_ResourceOwnerAccount(self):
        return self.get_query_params().get('ResourceOwnerAccount')

    def set_ResourceOwnerAccount(self,ResourceOwnerAccount):
        self.add_query_param('ResourceOwnerAccount',ResourceOwnerAccount)

    def get_TemplateParam(self):
        return self.get_query_params().get('TemplateParam')

    def set_TemplateParam(self,TemplateParam):
        self.add_query_param('TemplateParam',TemplateParam)

    def get_ResourceOwnerId(self):
        return self.get_query_params().get('ResourceOwnerId')

    def set_ResourceOwnerId(self,ResourceOwnerId):
        self.add_query_param('ResourceOwnerId',ResourceOwnerId)

    def get_OwnerId(self):
        return self.get_query_params().get('OwnerId')

    def set_OwnerId(self,OwnerId):
        self.add_query_param('OwnerId',OwnerId)

    def get_SmsUpExtendCode(self):
        return self.get_query_params().get('SmsUpExtendCode')

    def set_SmsUpExtendCode(self,SmsUpExtendCode):
        self.add_query_param('SmsUpExtendCode',SmsUpExtendCode)

    def get_OutId(self):
        return self.get_query_params().get('OutId')

    def set_OutId(self,OutId):
        self.add_query_param('OutId',OutId)


# 注意：不要更改
REGION = "cn-hangzhou"
PRODUCT_NAME = "Dysmsapi"
DOMAIN = "dysmsapi.aliyuncs.com"
ACCESS_KEY_ID = sd.config('aliyun_sms','key_id')
ACCESS_KEY_SECRET = sd.config('aliyun_sms','key_secret')

acs_client = AcsClient(ACCESS_KEY_ID, ACCESS_KEY_SECRET, REGION)
# region_provider.add_endpoint(PRODUCT_NAME, REGION, DOMAIN)
region_provider.modify_point(PRODUCT_NAME, REGION, DOMAIN)

def send_sms(phone_numbers, template_code, template_param=None):
    print(phone_numbers, template_code, template_param)
    business_id = uuid.uuid4()
    sign_name = sd.config('aliyun_sms','name')

    sms_request = SendSmsRequest()  # 申请的短信模板编码,必填
    sms_request.set_TemplateCode(template_code)  # 短信模板变量参数
    if template_param is not None:
        sms_request.set_TemplateParam(template_param)
    sms_request.set_OutId(business_id)  # 设置业务请求流水号，必填。
    sms_request.set_SignName(sign_name)  # 短信签名
    sms_request.set_PhoneNumbers(phone_numbers)  # 短信发送的号码列表，必填。
    sms_response = acs_client.do_action_with_exception(sms_request)  # 调用短信发送接口，返回json

    return sms_response

# if __name__ == '__main__':
#     params = {'code': 1, 'message': '模块{}一小时内错误数:{}, 请及时解决.'}
#     print(send_sms("18252064545", "SMS_154505884", params))
    #print(send_sms(__business_id, "13000000000", "云通信测试", "SMS_5250008", params))
